--[[
Group Repatch - v0.1.0
Created by Jason Giaffo
Contact: GiaffoDesigns.com/contact
Last updated May 1, 2017
--]]



---- USER CONFIG ----

local patch_warnings	= false	--set to true to get warnings before patching over existing fixtures; set to false to overwrite without confirmation


---- FUNCTIONS ----
local unpack = table.unpack

function getGroup(grpNum, inclSubs)
	--inclSubs parameter: if false, only direct fixture/channel numbers returned; if true, subfixtures returned in list
	
  gma.cmd('SelectDrive 1') --select the internal drive
  
  local file = {}
  file.name = 'tempFile.xml' 
  file.directory = gma.show.getvar('PATH')..'/'..'importexport'..'/'
  file.fullpath = file.directory..file.name
  
  gma.cmd('Export Group ' .. grpNum .. ' \"' .. file.name .. '\"') -- create temporary file
  
  local t = {}  --convert XML file into a table
  for line in io.lines(file.fullpath) do
    t[#t + 1] = line
  end
  os.remove(file.fullpath) --delete temporary file
  
  local groupList = {} --declare groupList
  for i = 1, #t do
    if t[i]:find('Subfixture ') then
	  local indices = {t[i]:find('\"%d+\"')} --find points of quotation marks
	  indices[1], indices[2] = indices[1] + 1, indices[2] - 1 --move reference points to first an last characters inside those marks
	  
	  --label based on status as a fixture or as a channel
	  local fixture
	  if t[i]:find('fix_id') then
	    fixture = 'Fixture ' .. tostring(t[i]:sub(unpack(indices))) --extract the number as the fixture number
	  elseif t[i]:find('cha_id') then
	    fixture = 'Channel ' .. tostring(t[i]:sub(unpack(indices))) end --extract the number as the fixture number
		
      --if the object contains a subfixture...
	  if inclSubs and t[i]:find('sub_index') then
	    local indices = {t[i]:find('\"%d+\"', indices[2]+2)}
		indices[1], indices[2] = indices[1] + 1, indices[2] - 1
		fixture = fixture .. '.' .. tostring(t[i]:sub(unpack(indices)))
      end
	  
	  --append to list
	  groupList[#groupList + 1] = fixture --extract the number to the group list
	end
  end
  
  return groupList
end



local function CSVlineToArray(str, header)
--convert a line of CSV text to an array
--indexed corresponding to position in provided header array
--i.e. if header[2] = 'title', then the second entry is array.title = value
--indexed numerically by position where no matching header slot exists, or if no header provided at all

--returns nil for all empty positions, NOT empty strings
  if not header then header = {} end
  
  local t = {}
  local indices = {1, 1}
  local remaining = true
  local restart = 1
  local index = 1
  while remaining == true do
    local index_ref = header[index]
    if not header[index] then index_ref = index	end --use loop ct as indexing value in returned table if header column is empty.
													--will be different from a column with the same number, because type will be number, not string
	
    local pt = {str:find(',', restart)}		--find next comma at or after restart point
    if pt[1] then
	  t[index_ref] = str:sub(restart, pt[1]-1)	--extract string from restart point to one position before comma
	  restart = pt[2] + 1 					--reset restart point for one position after most recent found comma
	  index = index + 1
    else
	  remaining = false
	  t[index_ref] = str:sub(restart, str[-1])
	end
  end
  
  for k, v in pairs(t) do
    if #v == 0 then t[k] = nil end
  end

  return t
end


local function CSVfileToArray(file, keepcase)
--pconverts a CSV file to an array, using the first line as a header by which to index all objects in the matching column below it
--uses CSVlineToArray function

--set keepcase argument to true to keep capital cases in the header; otherwise automatically converted to lowercase

	local header = {}
	local results = {}

	local firstLine = true
	for line in io.lines(file) do
	  if firstLine then
		header = CSVlineToArray(line)
		if (not keepcase) then
		  for k, v in pairs(header) do
		    if type(v) == 'string' then header[k] = string.lower(header[k])	end --convert all headers to lowercase to avoid confusion later unless otherwise specified
		  end
		end
		firstLine = false
	  else
		results[#results+1] = CSVlineToArray(line, header)
	  end
	end

	return results
  
end



local function address_to_array_dec(str)
  local index = str:find('[.]')
  local t = {}
  t[1] = str:sub(1, index-1)
  t[2] = str:sub(index+1, #str)

  return t
end



return function()
	-- define parameters for repatching process
	local msg = {}
	msg.def 		= 'Enter # of Group to repatch.'
	
	local group = {}
	group.src		= tonumber(gma.textinput('Group to Repatch?', msg.def))
	if not group.src or group.src < 1 then goto EOF end
	
	local univ = {}
	univ.dest		= tonumber(gma.textinput('Destination Universe?', '0'))
	if not univ.dest or univ.dest < 1 then goto EOF end
	
	univ.only		= tonumber(gma.textinput('Exclusive Univ to Repatch?', '0'))
	if univ.only == 0 then univ.only = nil end
	
	group.list = getGroup(group.src)
	group.export_str = group.list[1]
	for i = 2, #group.list do
		group.export_str = group.export_str..' + '..group.list[i]
	end
	
	--convert fixture list into a patch list
	local fn = 'tempfile_patch.csv'
	local dir = gma.show.getvar('PATH')..'/reports/'
	local filepath = dir..fn
	
	gma.cmd('SelectDrive 1')
	gma.cmd('List '..group.export_str..' /f=\"'..fn..'\"')
	local CSVlist = CSVfileToArray(filepath)
	--os.remove(filepath)
	
	-- create patch list
	local patch = {}
	patch.from, patch.to = {}, {}
	for i = 1, #CSVlist do
		if tonumber(CSVlist[i].patch) then
			gma.echo(i)																									--TROUBLESHOOT
			patch.from[i] = address_to_array_dec(CSVlist[i].patch)
			patch.to[i] = {univ.dest, patch.from[i][2]}
		else
			patch.from[i], patch.to[i] = {0, 0}, {0, 0}
		end
	end
	
	--repatch, IF source fixture is patched, and either exclusive universe was not specified, or was specified and is matched in source patch for fixture
	for i = 1, #group.list do
		gma.echo(i..' '..type(patch.to[i]))
		gma.echo(type(group.list[i]))
		if (patch.from[i][1] ~= 0) and (not univ.only) or (univ.only and tonumber(patch.from[i][1]) == univ.only) then
			gma.cmd('Assign '..group.list[i]..' /patch='..patch.to[i][1]..'.'..patch.to[i][2])
		end
	end
	
	::EOF::
end
